@@ -24,8 +24,11 @@
 
 #include <stdlib.h>
 #include <string.h>
+#include <sys/stat.h>
+#include <fcntl.h>
 #include <sys/types.h>
 #include <unistd.h>
+#include <errno.h>
 
 #include <glib.h>
 #include <glib/gi18n.h>
@@ -58,6 +61,10 @@ GtkWidget *totem_statusbar_create (void);
 GtkWidget *totem_volume_create (void);
 GtkWidget *totem_pp_create (void);
 
+/* Private function in totem-pl-parser, not for use
+ * by anyone but us */
+char * totem_pl_parser_resolve_uri (GFile *base_gfile, const char *relative_uri);
+
 #define VOLUME_DOWN_OFFSET (-0.08)
 #define VOLUME_UP_OFFSET (0.08)
 #define MINIMUM_VIDEO_SIZE 150
@@ -371,68 +378,16 @@ totem_embedded_set_state (TotemEmbedded *emb, TotemStates state)
 	emb->state = state;
 }
 
-static GdkPixbuf *
-totem_embedded_pad_pixbuf_for_size (GdkPixbuf *pixbuf,
-				    int width, int height)
-{
-	GdkPixbuf *logo;
-	guchar *pixels;
-	int rowstride, i;
-	int dest_x, dest_y;
-
-	logo = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
-			       TRUE, 8, width, height);
-	pixels = gdk_pixbuf_get_pixels (logo);
-	rowstride = gdk_pixbuf_get_rowstride (logo);
-
-	/* Clear it */
-	for (i = 0; i < height; i++) {
-		memset (pixels + i * rowstride, 0, width * 4);
-	}
-
-	dest_x = (width - gdk_pixbuf_get_width (pixbuf)) / 2;
-	dest_y = (height - gdk_pixbuf_get_height (pixbuf)) / 2;
-	gdk_pixbuf_copy_area (pixbuf,
-			      0, 0,
-			      MIN (gdk_pixbuf_get_width (logo), gdk_pixbuf_get_width (pixbuf)),
-			      MIN (gdk_pixbuf_get_height (logo), gdk_pixbuf_get_height (pixbuf)),
-			      logo,
-			      dest_x,
-			      dest_y);
-
-	return logo;
-}
-
 static void
 totem_embedded_set_logo_by_name (TotemEmbedded *embedded,
 				 const char *name)
 {
-	GtkIconTheme *theme;
-	GdkPixbuf *logo, *padded;
-	int size, width, height;
-
 	totem_embedded_set_state (embedded, TOTEM_STATE_STOPPED);
 
 	if (embedded->audioonly != FALSE || embedded->hidden != FALSE)
 		return;
 
-	theme = gtk_icon_theme_get_for_screen (gtk_widget_get_screen (embedded->window));
-
-	width = GTK_WIDGET (embedded->bvw)->allocation.width;
-	height = GTK_WIDGET (embedded->bvw)->allocation.height;
-	size = MIN (width, height);
-
-	logo = gtk_icon_theme_load_icon (theme, name, size, 0, NULL);
-	if (logo == NULL) {
-		g_warning ("Couldn't load '%s' icon from theme", name);
-		return;
-	}
-	padded = totem_embedded_pad_pixbuf_for_size (logo, width, height);
-	g_object_unref (logo);
-	if (padded != NULL) {
-		bacon_video_widget_set_logo_pixbuf (embedded->bvw, padded);
-		bacon_video_widget_set_logo_mode (embedded->bvw, TRUE);
-	}
+	bacon_video_widget_set_logo (embedded->bvw, name);
 }
 
 static void
@@ -474,7 +429,7 @@ totem_embedded_open_internal (TotemEmbedded *emb,
 
 	/* FIXME: remove |err| and rely on async on_error? */
 	g_message ("BEFORE _open");
-	retval = bacon_video_widget_open (emb->bvw, uri, &err);
+	retval = bacon_video_widget_open (emb->bvw, uri, NULL, &err);
 	g_message ("AFTER _open (ret: %d)", retval);
 	if (retval == FALSE)
 	{
@@ -661,102 +616,26 @@ totem_embedded_launch_player (TotemEmbedded *embedded,
 	return result;
 }
 
-static char *
-relative_uri_remove_query (const char *uri, char **query)
-{
-	char *qmark;
-
-	/* Look for '?' */
-	qmark = strrchr (uri, '?');
-	if (qmark == NULL)
-		return NULL;
-
-	*query = g_strdup (qmark);
-	return g_strndup (uri, qmark - uri);
-}
-
-static char *
-resolve_uri (const char *base_uri,
-	     const char *relative_uri)
-{
-	char *uri, *scheme, *query, *new_relative_uri;
-	GFile *base_gfile, *base_parent_gfile, *resolved_gfile;
-
-	if (!relative_uri)
-		return g_strdup (base_uri);
-
-	if (!base_uri)
-		return g_strdup (relative_uri);
-
-	/* If |relative_uri| has a scheme, it's a full URI, just return it */
-	scheme = g_uri_parse_scheme (relative_uri);
-	if (scheme) {
-		g_free (scheme);
-		return g_strdup (relative_uri);
-	}
-
-	base_gfile = g_file_new_for_uri (base_uri);
-	base_parent_gfile = g_file_get_parent (base_gfile);
-	if (!base_parent_gfile) {
-		g_print ("Base URI %s has no parent!\n", base_uri);
-		g_object_unref (base_gfile);
-		return NULL;
-	}
-	g_object_unref (base_gfile);
-
-	query = NULL;
-	new_relative_uri = relative_uri_remove_query (relative_uri, &query);
-
-	if (new_relative_uri) {
-		char *tmpuri;
-
-		resolved_gfile = g_file_resolve_relative_path (base_parent_gfile, new_relative_uri);
-		g_object_unref (base_parent_gfile);
-		if (!resolved_gfile) {
-			g_warning ("Failed to resolve relative URI '%s' against base '%s'\n", relative_uri, base_uri);
-			g_free (new_relative_uri);
-			g_free (query);
-			return NULL;
-		}
-
-		tmpuri = g_file_get_uri (resolved_gfile);
-		g_object_unref (resolved_gfile);
-		uri = g_strdup_printf ("%s%s", tmpuri, query);
-
-		g_free (tmpuri);
-		g_free (new_relative_uri);
-		g_free (query);
-
-		return uri;
-	} else {
-		resolved_gfile = g_file_resolve_relative_path (base_parent_gfile, relative_uri);
-		g_object_unref (base_parent_gfile);
-		if (!resolved_gfile) {
-			g_warning ("Failed to resolve relative URI '%s' against base '%s'\n", relative_uri, base_uri);
-			return NULL;
-		}
-
-		uri = g_file_get_uri (resolved_gfile);
-		g_object_unref (resolved_gfile);
-
-		return uri;
-	}
-}
-
 static void
 totem_embedded_set_uri (TotemEmbedded *emb,
 		        const char *uri,
 		        const char *base_uri,
 		        gboolean is_browser_stream)
 {
+	GFile *base_gfile;
 	char *old_uri, *old_base, *old_href;
 
+	base_gfile = NULL;
 	old_uri = emb->current_uri;
 	old_base = emb->base_uri;
 	old_href = emb->href_uri;
 
 	emb->base_uri = g_strdup (base_uri);
-	emb->current_uri = resolve_uri (base_uri, uri);
+	if (base_uri)
+		base_gfile = g_file_new_for_uri (base_uri);
+	emb->current_uri = totem_pl_parser_resolve_uri (base_gfile, uri);
+	if (base_gfile)
+		g_object_unref (base_gfile);
 	emb->is_browser_stream = (is_browser_stream != FALSE);
 	emb->href_uri = NULL;
 
@@ -765,6 +644,12 @@ totem_embedded_set_uri (TotemEmbedded *emb,
 	else
 		g_print ("Emptying current_uri\n");
 
+	if (old_uri != NULL &&
+	    g_str_has_prefix (old_uri, "fd://") != FALSE) {
+		int fd;
+		fd = strtol (old_uri + strlen ("fd://"), NULL, 0);
+		close (fd);
+	}
 	g_free (old_uri);
 	g_free (old_base);
 	g_free (old_href);
@@ -906,6 +791,9 @@ totem_embedded_open_uri (TotemEmbedded *emb,
 	totem_embedded_clear_playlist (emb, NULL);
 
 	totem_embedded_set_uri (emb, uri, base_uri, FALSE);
+	/* We can only have one item in the "playlist" when
+	 * we open a particular URI like this */
+	emb->num_items = 1;
 
 	return totem_embedded_open_internal (emb, TRUE, error);
 }
@@ -1021,18 +909,23 @@ totem_embedded_set_local_cache (TotemEmbedded *emb,
 				const char *path,
 				GError **error)
 {
-	char *file_uri;
+	int fd;
 
 	/* FIXME Should also handle playlists */
 	if (!emb->is_browser_stream)
 		return TRUE;
 
-	file_uri = g_filename_to_uri (path, NULL, error);
-	if (!file_uri)
+	/* Keep the temporary file open, so that StreamAsFile
+	 * doesn't remove it from under us */
+	fd = open (path, O_RDONLY);
+	if (fd < 0) {
+		g_message ("Failed to open local cache file '%s': %s",
+			   path, g_strerror (errno));
 		return FALSE;
+	}
 
 	emb->stream_uri = emb->current_uri;
-	emb->current_uri = file_uri;
+	emb->current_uri = g_strdup_printf ("fd://%d", fd);
 
 	return TRUE;
 }
@@ -1045,15 +938,47 @@ totem_embedded_set_playlist (TotemEmbedded *emb,
 			     GError **error)
 {
 	char *file_uri;
+	char *tmpfile;
+	GError *err = NULL;
+	GFile *src, *dst;
+	int fd;
 
 	g_message ("Setting the current playlist to %s (base: %s)",
 		   path, base_uri);
 
 	totem_embedded_clear_playlist (emb, NULL);
 
-	file_uri = g_filename_to_uri (path, NULL, error);
-	if (!file_uri)
-		return FALSE;
+	/* FIXME, we should remove that when we can
+	 * parse from memory or 
+	 * https://bugzilla.gnome.org/show_bug.cgi?id=598702 is fixed */
+	fd = g_file_open_tmp ("totem-browser-plugin-playlist-XXXXXX",
+			      &tmpfile,
+			      &err);
+	if (fd < 0) {
+		g_warning ("Couldn't open temporary file for playlist: %s",
+			   err->message);
+		g_error_free (err);
+		return TRUE;
+	}
+	src = g_file_new_for_path (path);
+	dst = g_file_new_for_path (tmpfile);
+	if (g_file_copy (src, dst, G_FILE_COPY_OVERWRITE, NULL, NULL, NULL, &err) == FALSE) {
+		g_warning ("Failed to copy playlist '%s' to '%s': %s",
+			   path, tmpfile, err->message);
+		g_error_free (err);
+		g_object_unref (src);
+		g_object_unref (dst);
+		g_free (tmpfile);
+		close (fd);
+		return TRUE;
+	}
+	g_free (tmpfile);
+
+	file_uri = g_file_get_uri (dst);
+
+	g_object_unref (src);
+	g_object_unref (dst);
+	close (fd);
 
 	totem_embedded_set_uri (emb, file_uri, base_uri, FALSE);
 	g_free (file_uri);
@@ -2135,6 +2060,7 @@ totem_embedded_push_parser (gpointer data)
 	TotemEmbedded *emb = (TotemEmbedded *) data;
 	TotemPlParser *parser;
 	TotemPlParserResult res;
+	GFile *file;
 
 	emb->parser_id = 0;
 
@@ -2147,6 +2073,12 @@ totem_embedded_push_parser (gpointer data)
 					       emb->base_uri, FALSE);
 	g_object_unref (parser);
 
+	/* Delete the temporary file created in
+	 * totem_embedded_set_playlist */
+	file = g_file_new_for_uri (emb->current_uri);
+	g_file_delete (file, NULL, NULL);
+	g_object_unref (file);
+
 	if (res != TOTEM_PL_PARSER_RESULT_SUCCESS) {
 		//FIXME show a proper error message
 		switch (res) {
@@ -2259,6 +2191,9 @@ int main (int argc, char **argv)
 
 	g_thread_init (NULL);
 
+	g_set_application_name (_("Movie browser plugin"));
+	gtk_window_set_default_icon_name ("totem");
+
 #ifdef GNOME_ENABLE_DEBUG
 	{
 		int i;
@@ -2272,46 +2207,46 @@ int main (int argc, char **argv)
 	if (XInitThreads () == 0)
 	{
 		gtk_init (&argc, &argv);
-		totem_embedded_error_and_exit (_("Could not initialise the thread-safe libraries."), _("Verify your system installation. The Totem plugin will now exit."), NULL);
+		totem_embedded_error_and_exit (_("Could not initialize the thread-safe libraries."), _("Verify your system installation. The Totem plugin will now exit."), NULL);
 	}
 
 	dbus_g_thread_init ();
 
 #ifdef GNOME_ENABLE_DEBUG
-{
-	const char *env;
+	{
+		const char *env;
 
-	env = g_getenv ("TOTEM_EMBEDDED_GDB");
-	if (env && g_ascii_strtoull (env, NULL, 10) == 1) {
-		char *gdbargv[6];
-		char pid[32];
-		GError *gdberr = NULL;
-		int gdbargc = 0;
+		env = g_getenv ("TOTEM_EMBEDDED_GDB");
+		if (env && g_ascii_strtoull (env, NULL, 10) == 1) {
+			char *gdbargv[6];
+			char pid[32];
+			GError *gdberr = NULL;
+			int gdbargc = 0;
 
-		g_snprintf (pid, sizeof (pid), "%d", getpid ());
+			g_snprintf (pid, sizeof (pid), "%d", getpid ());
 
-		gdbargv[gdbargc++] = "/usr/bin/xterm";
-		gdbargv[gdbargc++] = "-e";
-		gdbargv[gdbargc++] = "gdb";
-		gdbargv[gdbargc++] = argv[0];
-		gdbargv[gdbargc++] = pid;
-		gdbargv[gdbargc++] = NULL;
+			gdbargv[gdbargc++] = "/usr/bin/xterm";
+			gdbargv[gdbargc++] = "-e";
+			gdbargv[gdbargc++] = "gdb";
+			gdbargv[gdbargc++] = argv[0];
+			gdbargv[gdbargc++] = pid;
+			gdbargv[gdbargc++] = NULL;
 
-		if (!g_spawn_async (NULL,
-				    gdbargv,
-				    NULL /* env */,
-				    0,
-				    NULL, NULL,
-				    NULL,
-				    &gdberr)) {
-			g_warning ("Failed to spawn debugger: %s", gdberr->message);
-			g_error_free (gdberr);
-		} else {
-			g_print ("Sleeping....\n");
-			g_usleep (10* 1000 * 1000); /* 10s */
+			if (!g_spawn_async (NULL,
+					    gdbargv,
+					    NULL /* env */,
+					    0,
+					    NULL, NULL,
+					    NULL,
+					    &gdberr)) {
+				g_warning ("Failed to spawn debugger: %s", gdberr->message);
+				g_error_free (gdberr);
+			} else {
+				g_print ("Sleeping....\n");
+				g_usleep (10* 1000 * 1000); /* 10s */
+			}
 		}
 	}
-}
 #endif
 
 	if (!gtk_init_with_args (&argc, &argv, NULL, option_entries, GETTEXT_PACKAGE, &e))
@@ -2321,6 +2256,11 @@ int main (int argc, char **argv)
 		exit (1);
 	}
 
+	if (arg_audioonly != FALSE)
+		g_setenv("PULSE_PROP_media.role", "video", TRUE);
+	else
+		g_setenv("PULSE_PROP_media.role", "music", TRUE);
+
         // FIXME check that ALL necessary params were given!
 	if (arg_plugin_type == TOTEM_PLUGIN_TYPE_LAST) {
 		g_warning ("Plugin type is required\n");
@@ -2329,10 +2269,8 @@ int main (int argc, char **argv)
 
 	/* FIXME: check the UA strings of the legacy plugins themselves */
 	/* FIXME: at least hxplayer seems to send different UAs depending on the protocol!? */
-	/* FIXME: this won't work with gvfs:
-	 * http://bugzilla.gnome.org/show_bug.cgi?id=534482 */
 	if (arg_user_agent != NULL) {
-		g_setenv ("GNOME_VFS_HTTP_USER_AGENT", arg_user_agent, TRUE);
+		g_setenv ("BACON_VIDEO_WIDGET_HTTP_USER_AGENT", arg_user_agent, TRUE);
 		g_free (arg_user_agent);
 		arg_user_agent = NULL;
 	}
