@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  * The Totem project hereby grant permission for non-gpl compatible GStreamer
  * plugins to be used and distributed together with GStreamer and Totem. This
@@ -72,9 +72,8 @@ void     totem_fullscreen_set_video_widget   (TotemFullscreen *fs,
 					      BaconVideoWidget *bvw);
 void     totem_fullscreen_set_parent_window  (TotemFullscreen *fs,
 					      GtkWindow *parent_window);
-gboolean totem_fullscreen_motion_notify      (GtkWidget *widget,
-					      GdkEventMotion *event,
-					      TotemFullscreen *fs);
+void     totem_fullscreen_show_popups        (TotemFullscreen *fs,
+					      gboolean show_cursor);
 gboolean totem_fullscreen_is_fullscreen      (TotemFullscreen *fs);
 void     totem_fullscreen_set_fullscreen     (TotemFullscreen *fs,
 					      gboolean fullscreen);
